package com.tenlionsoft.aimz_k

import android.util.Log
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.model.CoverSealedBean
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.aimz_k.model.MsgConvertBean
import com.tenlionsoft.aimz_k.model.MsgTypeStateEnum
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.SpUtils

object ConvertBeanUtils {

    fun covertBean(str: String): CoverSealedBean {
//        {"body":"{\"msg\":\"\",\"statusType\":\"SUCCESS_SEND\"}",
        //        "customMessageType":"",
        //        "messageId":"1731568391763",
        //        "messageType":"STATUS",
        //        "metadata":"",
        //        "receiver":{"receiverId":"c7b7ca3b-1828-4cdf-aa59-5e8efacd6939","receiverType":"SINGLE_USER"},
        //        "sender":{"senderId":"system","senderType":"SYSTEM"},
        //        "timestamp":1731568386979}
        val gson = Gson()
        val sBean = gson.fromJson(str, MsgConvertBean::class.java)
        Log.e("ConvertBeanUtils", "covertBean:${sBean}")
        var type: Int = -1
        if (sBean.messageType != ProjectConfig.MSG_STATUS) {
            when (sBean.messageType) {
                ProjectConfig.MSG_TEXT -> type = MsgTypeStateEnum.MSG_FROM_OTHER_TXT
                ProjectConfig.MSG_IMG -> type = MsgTypeStateEnum.MSG_FROM_OTHER_IMG
                ProjectConfig.MSG_FILE -> type = MsgTypeStateEnum.MSG_FROM_OTHER_FILE
                ProjectConfig.MSG_VIDEO -> type = MsgTypeStateEnum.MSG_FROM_OTHER_MOVIE
            }
            return CoverSealedBean.Msg(
                MsgBean(
                    messageId = sBean.messageId,
                    messageType = sBean.messageType,
                    metadata = sBean.metadata,
                    receiverId = sBean.sender.senderId,
                    receiverType = sBean.sender.senderType,
                    senderType = sBean.receiver.receiverType,
                    senderId = sBean.receiver.receiverId,
                    body = sBean.body,
                    msgType = type,
                    timestamp = sBean.timestamp,
                    status = sBean.status,
                    customMessageType = "",
                    userNickName = "",
                    userAvatar = "",
                    loginUserId = SpUtils.getId(),
                    receiverAvatar = SpUtils.getAvatar(),
                    receiverNickName = SpUtils.getNickName(),
                    senderAvatar = "",
                    senderNickName = ""
                )
            )
        } else {
            return CoverSealedBean.StateBean(sBean)
        }
    }

    fun convertBeanToMsgBean(cBean: MsgConvertBean): MsgBean {
        var type: Int = -1
        when (cBean.messageType) {
            ProjectConfig.MSG_TEXT -> type = MsgTypeStateEnum.MSG_TO_OTHER_TXT
            ProjectConfig.MSG_IMG -> type = MsgTypeStateEnum.MSG_TO_OTHER_IMG
            ProjectConfig.MSG_FILE -> type = MsgTypeStateEnum.MSG_TO_OTHER_FILE
            ProjectConfig.MSG_VIDEO -> type = MsgTypeStateEnum.MSG_TO_OTHER_MOVIE
        }
        return MsgBean(
            messageId = cBean.messageId,
            messageType = cBean.messageType,
            metadata = cBean.metadata,
            receiverId = cBean.receiver.receiverId,
            receiverType = cBean.receiver.receiverType,
            senderType = cBean.sender.senderType,
            senderId = cBean.sender.senderId,
            body = cBean.body,
            msgType = type,
            timestamp = cBean.timestamp,
            status = cBean.status,
            customMessageType = "",
            userNickName = "",
            userAvatar = SpUtils.getAvatar(),
            loginUserId = SpUtils.getId(),
            senderAvatar = SpUtils.getAvatar(),
            senderNickName = SpUtils.getNickName(),
            receiverAvatar = "",
            receiverNickName = ""
        )
    }

    fun convertCategoryBean(msgBean: MsgBean): MsgCategoryBean {
        return MsgCategoryBean(
            avatar = "",
            messageType = msgBean.messageType,
            messageId = msgBean.messageId,
            receiverId = msgBean.receiverId,
            senderId = msgBean.senderId,
            status = msgBean.status,
            timestamp = msgBean.timestamp,
            msgType = msgBean.msgType,
            body = msgBean.body,
            userNickName = "",
            loginUserId = SpUtils.getId(),
        )
    }

}