package com.tenlionsoft.aimz_k.net

import com.tenlionsoft.aimz_k.model.ApplyContactBean
import com.tenlionsoft.aimz_k.model.BaseResponseBean
import com.tenlionsoft.aimz_k.model.FileUploadStateBean
import com.tenlionsoft.aimz_k.model.PageListBean
import com.tenlionsoft.aimz_k.model.SearchContactBean
import com.tenlionsoft.aimz_k.model.SelfContactBean
import com.tenlionsoft.aimz_k.model.UserInfoBean
import com.tenlionsoft.baselib.model.VersionBean
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.http.Body
import retrofit2.http.DELETE
import retrofit2.http.GET
import retrofit2.http.Headers
import retrofit2.http.Multipart
import retrofit2.http.POST
import retrofit2.http.PUT
import retrofit2.http.Part
import retrofit2.http.Path
import retrofit2.http.Query


interface BaseApi {
    /**
     * 登录
     *
     * @param user
     * @return
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "projectName:usercenter")
    @POST("api/jwt/login")
    suspend fun doLogin(
        @Body user: RequestBody
    ): BaseResponseBean

    /**
     * 根据ID获取用户信息
     */
    @Headers("token:need", "Content-Type: application/json", "Accept: application/json")
    @GET("api/user/get/uuid/{uuid}")
    suspend fun getUserInfo(
        @Path("uuid") uId: String
    ): UserInfoBean?

    /**
     * 登陆Socket系统
     */
    @Headers("Content-Type: application/json", "Accept: application/json")
    @POST("api/anonymous/login")
    suspend fun doLoginSocket(
        @Body user: RequestBody
    ): BaseResponseBean

    /**
     * 获取App版本
     *
     * @return
     */
    @Headers("token:need", "Content-Type: application/json", "Accept: application/json")
    @GET("app/appversion/get-number/{appVersionId}")
    suspend fun doCheckAppVersion(
        @Path("appVersionId") appId: String
    ): VersionBean


    /**
     * 上传图片
     * image
     *
     * @param file
     * @param token
     * @return
     */
    @Headers("token:need")
    @Multipart
    @POST("api/file/upload/image")
    suspend fun doUploadImage(
        @Part file: MultipartBody.Part
    ): FileUploadStateBean

    /**
     * 更新用户头像
     */
    @Headers("token:need")
    @PUT("api/user/update-avatar/uuid/{uuid}")
    suspend fun doUpdateUserAvatar(
        @Path("uuid") uuid: String, @Body body: RequestBody
    ): BaseResponseBean

    /**
     * 更新用户信息
     */
    @Headers("token:need")
    @PUT("api/user/update-base-info/uuid/{uuid}")
    suspend fun doUpdateUserInfo(
        @Path("uuid") uuid: String, @Body body: RequestBody
    ): BaseResponseBean

    /**
     * 上传视频
     * video
     *
     * @param file
     * @param token
     * @return
     */
    @Headers("token:need")
    @Multipart
    @POST("api/file/upload/video")
    suspend fun doUploadVideo(
        @Part file: MultipartBody.Part
    ): FileUploadStateBean

    /**
     * 上传文件
     */
    @Headers("token:need")
    @Multipart
    @POST("api/file/upload/file")
    suspend fun doUploadFile(
        @Part file: MultipartBody.Part
    ): FileUploadStateBean

    /**
     * 上传音频
     */
    @Headers("token:need")
    @Multipart
    @POST("api/file/upload/audio")
    suspend fun doUploadAudio(
        @Part file: MultipartBody.Part
    ): FileUploadStateBean

    /**
     * 获取我的联系人列表
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "token:need")
    @GET("api/contact/user/list/self")
    suspend fun getMineContactList(): List<SelfContactBean>

    /**
     * 搜索联系人
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "token:need")
    @GET("api/user/listpage")
    suspend fun doSearchContact(
        @Query("keyword") keyword: String
    ): PageListBean<SearchContactBean>

    /**
     * 新增联系人
     * doAddContact
     *
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "token:need")
    @POST("api/contact/user/apply/save")
    suspend fun doAddContact(
        @Body body: RequestBody
    ): BaseResponseBean

    /**
     * 获取好友申请人列表
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "token:need")
    @GET("api/contact/user/apply/list-review/self")
    suspend fun doGetMyApplyContactList(
    ): MutableList<ApplyContactBean>

    /**
     * 通过申请
     *
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "token:need")
    @PUT("api/contact/user/apply/update-status/self/uuid/{uuid}/status/{status}")
    suspend fun doPassApplyContact(
        @Path("uuid") id: String, @Path("status") status: String
    ): BaseResponseBean

    /**
     * 删除联系人
     * /
     */
    @Headers("Content-Type: application/json", "Accept: application/json", "token:need")
    @DELETE("api/contact/user/remove/uuids/{uuids}/self")
    suspend fun doDelContact(
        @Path("uuids") id: String
    ): BaseResponseBean
}