package com.tenlionsoft.aimz_k.page.activity

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.util.Log
import android.view.View
import android.view.WindowManager
import androidx.core.content.ContextCompat
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.LinearLayoutManager
import com.atwa.filepicker.core.FilePicker
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.ActivityChatBinding
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.FileDataBean
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.PickerType
import com.tenlionsoft.aimz_k.viewmodel.ChatPageViewModel
import com.tenlionsoft.baselib.base.BaseActivity
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.model.ModelEvent
import com.tenlionsoft.baselib.net.DownloadResponseHandler
import com.tenlionsoft.baselib.net.FileDownloadUtils
import com.tenlionsoft.baselib.utils.DensityUtils
import com.tenlionsoft.baselib.utils.FileUtils
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.StorageUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import com.tenlionsoft.baselib.widget.LoadingDialog
import com.tenlionsoft.baselib.widget.SoftKeyBoardListener
import com.tenlionsoft.baselib.widget.wheel.WheelView
import com.tenlionsoft.medialib.base.SimplePhotoActivity
import com.tenlionsoft.medialib.base.SimpleVideoActivity
import java.io.File


/**
 * 聊天页面
 */
class ChatActivity : BaseActivity(), AdapterItemClickListener<MsgBean> {
    private lateinit var mBinding: ActivityChatBinding
    private lateinit var mLocalReceiver: LocalReceiver
    var chatPageViewModel: ChatPageViewModel? = null
    private val filePicker = FilePicker.getInstance(this)
    private var mContentHeight: Int = 0
    private var mLoading: LoadingDialog? = null
    private var bottomHeight = -1
    val replyList = arrayListOf<String>()
    override fun bindView() {
        mBinding = DataBindingUtil.setContentView(this, R.layout.activity_chat)
        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
        window.statusBarColor =
            ContextCompat.getColor(this, com.tenlionsoft.baselib.R.color.chat_page_bg)

        val fromId = intent.getStringExtra("fromId") //传递过来的接收人
        val name = intent.getStringExtra("name")
        mBinding.tvTitle.text = if (name.isNullOrEmpty()) "匿名用户" else name

        chatPageViewModel = ViewModelProvider(this, object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return ChatPageViewModel(
                    fromId!!, SpUtils.getId(), this@ChatActivity.application
                ) as T
            }
        })[ChatPageViewModel::class.java]
        chatPageViewModel!!.onItemClickListener = this
        mBinding.ivBack.setOnClickListener { finish() }
        mBinding.viewModel = chatPageViewModel
        mBinding.lifecycleOwner = this
        //选择emoji
        mBinding.rpvEmoji.setOnEmojiPickedListener {
            mBinding.etMsg.append(it.emoji)
        }

        chatPageViewModel!!.showReplyLayout.observe(this) {
            if (it) {
                hideSoftKeyboard()//隐藏软键盘
                setContentLayoutHeight()
            } else {
                val isShow =
                    chatPageViewModel!!.showEmojiLayout.value!! || chatPageViewModel!!.showChooseLayout.value!!
                if (isShow) {
                    setContentDefaultHeight()
                }
            }
        }
        chatPageViewModel!!.showEmojiLayout.observe(this) {
            if (it) {
                hideSoftKeyboard()//隐藏软键盘
                setContentLayoutHeight()
            } else {
                val isShow =
                    chatPageViewModel!!.showReplyLayout.value!! || chatPageViewModel!!.showChooseLayout.value!!
                if (isShow) {
                    setContentDefaultHeight()
                }
            }
        }
        chatPageViewModel!!.showChooseLayout.observe(this) {
            if (it) {
                hideSoftKeyboard()//隐藏软键盘
                setContentLayoutHeight()
            } else {
                val isShow =
                    chatPageViewModel!!.showEmojiLayout.value!! || chatPageViewModel!!.showReplyLayout.value!!
                if (isShow) {
                    setContentDefaultHeight()
                }
            }
        }
        chatPageViewModel!!.showLoad.observe(this) {
            if (it.isShow) {
                mLoading = LoadingDialog.Builder(this).setCancelOutside(false).setCancelable(false)
                    .setMessage(it.data.toString()).create()
                mLoading!!.show()
            } else {
                mLoading?.dismiss()
                mLoading = null
            }
        }
        chatPageViewModel!!.showLoadDialog.observe(this) {
            if (it) {
                //显示loading
                mLoading = LoadingDialog.Builder(this).setCancelOutside(false).setCancelable(false)
                    .setMessage("上传中...").create()
                mLoading!!.show()
            } else {
                //隐藏loading
                mLoading?.dismiss()
                mLoading = null
            }
        }
        mBinding.wvView.setTextSize(16F, isSp = true)
        mBinding.wvView.setAutoFitTextSize(true)
        mBinding.wvView.setOnItemSelectedListener(object : WheelView.OnItemSelectedListener {
            override fun onItemSelected(wheelView: WheelView, data: Any, position: Int) {
                mBinding.etMsg.setText(data.toString())
            }
        })

        chatPageViewModel!!.chooseType.observe(this) {
            when (it) {
                PickerType.TYPE_PIC -> {
                    filePicker.pickMultiImage { list ->
                        if (list.isNotEmpty()) {
                            chatPageViewModel!!.uploadImages(list)
                        }
                    }
                }

                PickerType.TYPE_VIDEO -> {
                    filePicker.pickVideo { mate ->
                        if (mate != null) {
                            chatPageViewModel!!.uploadVideo(mate)
                        }
                    }
                }

                PickerType.TYPE_FILE -> {
                    filePicker.pickFile { meta ->
                        if (meta != null) {
                            chatPageViewModel!!.uploadFiles(meta)
                        }
                    }
                }

                else -> {}
            }
        }
        chatPageViewModel!!.scrollListToBottom.observe(this) {

            if (it) {
                val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
                val itemCount = layoutManager.itemCount
                //滚动到底部

                mBinding.rlvChats.postDelayed({
                    layoutManager.scrollToPositionWithOffset(itemCount - 1, 0)
                }, 50)
                chatPageViewModel!!.scrollListToBottom.value = false
            }
        }
        mBinding.rlBottom.visibility = View.GONE
        mBinding.rlvChats.viewTreeObserver.addOnGlobalLayoutListener {
            mContentHeight = mBinding.rlvChats.height
        }

        mBinding.llBottomLayout.viewTreeObserver.addOnGlobalLayoutListener {
            bottomHeight = if (mBinding.llBottomLayout.height == 0) {
                DensityUtils.dp2px(this@ChatActivity, 300F)
            } else {
                mBinding.llBottomLayout.height
            }
        }
        SoftKeyBoardListener().setChangeListener(this@ChatActivity, { h ->
            chatPageViewModel!!.showReplyLayout.value = false
            chatPageViewModel!!.showEmojiLayout.value = false
            chatPageViewModel!!.showChooseLayout.value = false
            mBinding.rlBottom.visibility = View.VISIBLE
            //重置layout高度
            val layoutParams = mBinding.rlvChats.layoutParams
            layoutParams.height = mContentHeight - h
            mBinding.rlvChats.layoutParams = layoutParams
            val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
            val itemCount = layoutManager.itemCount
            //滚动到底部
            mBinding.rlvChats.postDelayed({
                layoutManager.scrollToPositionWithOffset(itemCount - 1, 0)
            }, 100)
        }, { _ ->
            val isShowOther =
                chatPageViewModel!!.showReplyLayout.value!! || chatPageViewModel!!.showEmojiLayout.value!! || chatPageViewModel!!.showChooseLayout.value!!
            if (!isShowOther) {
                val layoutParams = mBinding.rlvChats.layoutParams
                layoutParams.height = mContentHeight
                mBinding.rlvChats.layoutParams = layoutParams
                mBinding.rlBottom.visibility = View.GONE
            }
        })

        chatPageViewModel!!._replyList.observe(this) { it ->
            if (it.isNotEmpty()) {
                replyList.clear()
                it.forEach {
                    replyList.add(it.content)
                }
                mBinding.wvView.setDataItems(replyList)
            }
        }
        registerLocalReceiver()
    }

    override fun onResume() {
        super.onResume()
        chatPageViewModel!!.getReplyData()
    }


    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    private fun registerLocalReceiver() {
        mLocalReceiver = LocalReceiver()
        val intentFilter = IntentFilter()
        intentFilter.addAction(ProjectConfig.A_S_UPDATE_INFO)
        intentFilter.addAction(ProjectConfig.A_S_UPDATE_STATUS)
        registerReceiver(mLocalReceiver, intentFilter)
    }

    private fun unRegisterLocalReceiver() {
        unregisterReceiver(mLocalReceiver)
    }

    private fun setContentLayoutHeight() {
        mBinding.rlBottom.visibility = View.VISIBLE
        //重置layout高度
        val layoutParams = mBinding.rlvChats.layoutParams
        layoutParams.height = mContentHeight - bottomHeight
        mBinding.rlvChats.layoutParams = layoutParams
        val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
        val itemCount = layoutManager.itemCount
        //滚动到底部
        mBinding.rlvChats.postDelayed({
            layoutManager.scrollToPositionWithOffset(itemCount - 1, 0)
        }, 100)
    }

    private fun setContentDefaultHeight() {
        val layoutParams = mBinding.rlvChats.layoutParams
        layoutParams.height = mContentHeight
        mBinding.rlvChats.layoutParams = layoutParams
        mBinding.rlBottom.visibility = View.GONE
    }

    override fun onDestroy() {
        super.onDestroy()
        unRegisterLocalReceiver()
    }

    inner class LocalReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            when (intent?.action) {
                ProjectConfig.A_S_UPDATE_INFO -> {
                    chatPageViewModel?.doRefresh()
                }

                ProjectConfig.A_S_UPDATE_STATUS -> {
                    val id = intent.getStringExtra(ProjectConfig.MSG_M_ID)
                    val status = intent.getStringExtra(ProjectConfig.MSG_M_STATUS)
                    if (!id.isNullOrEmpty() && !status.isNullOrEmpty()) {
                        chatPageViewModel?.doUpdateMsgStatus(id, status)
                    }
                }
            }
        }
    }

    //条目点击
    override fun onItemClick(data: MsgBean) {
        when (data.messageType) {
            ProjectConfig.MSG_TEXT -> {}
            ProjectConfig.MSG_IMG -> {
                val intent = Intent(this@ChatActivity, SimplePhotoActivity::class.java)
                val gson = Gson()
                val body = gson.fromJson(data.body, BodyContent::class.java)
                val fileDataBean = gson.fromJson(body.content, FileDataBean::class.java)
                val url = NetConfig.MAIN_URL + fileDataBean.fileUrl
                val urls = arrayListOf(url)
                intent.putStringArrayListExtra(SimplePhotoActivity.TAG_IMGURL, urls)
                startActivity(intent)
            }

            ProjectConfig.MSG_FILE -> {
                val gson = Gson()
                val body = gson.fromJson(data.body, BodyContent::class.java)
                val fileDataBean = gson.fromJson(body.content, FileDataBean::class.java)
                val dir: File = StorageUtils.getExternalCacheCustomDir(this)!!
                chatPageViewModel?.showLoad?.value = ModelEvent(true, "下载中...")
                FileDownloadUtils.getInstance().download(this@ChatActivity,
                    NetConfig.MAIN_URL + fileDataBean.fileUrl,
                    dir.absolutePath,
                    fileDataBean.fileName!!,
                    object : DownloadResponseHandler() {
                        override fun onFinish(downloadFile: File?) {
                            chatPageViewModel?.showLoad?.value = ModelEvent(false, "下载中...")
                            Log.e("ChatActivity", "onFinish:${downloadFile}")
                            FileUtils.openFile(this@ChatActivity, downloadFile)
                        }

                        override fun onProgress(currentBytes: Long, totalBytes: Long) {

                        }

                        override fun onFailure(errorMsg: String?) {
                            chatPageViewModel?.showLoad?.value = ModelEvent(false, "下载中...")
                            ToastUtils.error(
                                errorMsg
                                    ?: "下载失败,请稍后重试"
                            )
                        }
                    })
            }

            ProjectConfig.MSG_VIDEO -> {
                val intent = Intent(this@ChatActivity, SimpleVideoActivity::class.java)
                val gson = Gson()
                val body = gson.fromJson(data.body, BodyContent::class.java)
                val fileDataBean = gson.fromJson(body.content, FileDataBean::class.java)
                intent.putExtra("url", fileDataBean.fileUrl)
                intent.putExtra("title", fileDataBean.fileName)
                startActivity(intent)
            }
        }

    }
}


