package com.tenlionsoft.aimz_k.page.activity

import android.content.Intent
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.lifecycleScope
import com.hjq.permissions.OnPermissionCallback
import com.hjq.permissions.Permission
import com.hjq.permissions.XXPermissions
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.dao.DbManager
import com.tenlionsoft.aimz_k.databinding.ActivitySplashBinding
import com.tenlionsoft.aimz_k.model.ReplyBean
import com.tenlionsoft.baselib.base.BaseActivity
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.util.Timer
import java.util.TimerTask

/**
 * 闪屏页
 */
class SplashActivity : BaseActivity() {
    private lateinit var mBinding: ActivitySplashBinding


    override fun bindView() {
        mBinding = DataBindingUtil.setContentView(this, R.layout.activity_splash)
        mBinding.lifecycleOwner = this
        XXPermissions.with(this).permission(Permission.MANAGE_EXTERNAL_STORAGE)
            .request(object : OnPermissionCallback {
                override fun onGranted(permissions: MutableList<String>, allGranted: Boolean) {
                    if (allGranted) {
                        lifecycleScope.launch {
                            withContext(Dispatchers.IO) {
                                val msgDao = DbManager.db.msgDao()
                                val cDao = DbManager.db.categoryDao()
                                val rDao = DbManager.db.replayDao()

                                msgDao.queryContent("", SpUtils.getId())
                                cDao.getAllCategory()
                                if (rDao.getAllReply().size <= 0) {
                                    val replayList = arrayListOf(
                                        ReplyBean(
                                            content = "您好，很高兴为您服务!请问有什么可以帮到您的呢?",
                                            remark = ""
                                        ),
                                        ReplyBean(
                                            content = "感谢您的咨询，希望我的回答对您有所帮助，祝您生活愉快!",
                                            remark = ""
                                        ),
                                        ReplyBean(
                                            content = "希望您每天都有好心情，拜拜!",
                                            remark = ""
                                        ),
                                        ReplyBean(
                                            content = "请参考操作手册.",
                                            remark = ""
                                        )
                                    )
                                    rDao.insertAll(replayList)
                                }
                            }
                            Timer().schedule(object : TimerTask() {
                                override fun run() {
                                    startActivity(
                                        Intent(
                                            this@SplashActivity, LoginActivity::class.java
                                        )
                                    );
                                    finish();
                                }

                            }, 500)
                        }
                    } else {
                        ToastUtils.error("你未授权管理文件权限,无法使用软件")
                    }
                }

                override fun onDenied(permissions: MutableList<String>, doNotAskAgain: Boolean) {
                    if (doNotAskAgain) {
                        ToastUtils.error("被永久拒绝授权，请手动授予")
                        // 如果是被永久拒绝就跳转到应用权限系统设置页面
                        XXPermissions.startPermissionActivity(this@SplashActivity, permissions)
                    } else {
                        ToastUtils.error("获取权限失败")
                        finish()
                    }
                }
            })
    }
}