package com.tenlionsoft.aimz_k.page.activity

import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import com.atwa.filepicker.core.FilePicker
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.ActivityUserInfoEditBinding
import com.tenlionsoft.aimz_k.viewmodel.UserInfoEditViewModel
import com.tenlionsoft.baselib.base.BaseActivity
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.widget.LoadingDialog
import jp.wasabeef.glide.transformations.RoundedCornersTransformation

class UserInfoEditActivity : BaseActivity() {
    private var mLoading: LoadingDialog? = null
    private lateinit var mBind: ActivityUserInfoEditBinding
    private lateinit var viewModel: UserInfoEditViewModel
    private val filePicker = FilePicker.getInstance(this)
    override fun bindView() {
        mBind = DataBindingUtil.setContentView(
            this@UserInfoEditActivity,
            R.layout.activity_user_info_edit
        )
        viewModel = ViewModelProvider(this, object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return UserInfoEditViewModel(this@UserInfoEditActivity.application) as T
            }
        })[UserInfoEditViewModel::class.java]

        mBind.model = viewModel
        mBind.lifecycleOwner = this
        mBind.ivIcon.setOnClickListener {
            filePicker.pickImage { meta ->
                if (meta != null) {
                    viewModel.doUploadImg(meta)
                }
            }
        }
        viewModel.showLoad.observe(this) {
            if (it.isShow) {
                //显示loading
                mLoading = LoadingDialog.Builder(this)
                    .setCancelOutside(false)
                    .setCancelable(false)
                    .setMessage(it.data.toString())
                    .create()
                mLoading!!.show()
            } else {
                //隐藏loading
                mLoading?.dismiss()
                mLoading = null
            }
        }
        mBind.ivBack.setOnClickListener { finish() }
        val requestOptions = RequestOptions.bitmapTransform(RoundedCornersTransformation(10, 0))
            .error(R.drawable.ic_user_default)
            .placeholder(R.drawable.ic_user_default)
            .skipMemoryCache(false)
            .diskCacheStrategy(DiskCacheStrategy.RESOURCE)
        val icon = SpUtils.getAvatar().split(":")
        Glide.with(this@UserInfoEditActivity)
            .load(NetConfig.FILE_DOWNLOAD_URL + icon[0])
            .apply(requestOptions)
            .into(mBind.ivIcon)
        viewModel.isUpload.observe(this) {
            if (it) {
                val icon = SpUtils.getAvatar().split(":")
                Glide.with(this@UserInfoEditActivity)
                    .load(NetConfig.FILE_DOWNLOAD_URL + icon[0])
                    .apply(requestOptions)
                    .into(mBind.ivIcon)
            }
        }
    }
}