package com.tenlionsoft.aimz_k.viewmodel

import android.content.Context
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.atwa.filepicker.result.ImageMeta
import com.tenlionsoft.aimz_k.model.UserAvatarBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.contacts.toBody
import com.tenlionsoft.baselib.contacts.toFileBody
import com.tenlionsoft.baselib.net.ExParse
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.SpUtils
import kotlinx.coroutines.launch

class MineViewModel(val context: Context) : BaseViewModel() {
    private val retrofitClient = RetrofitClient.getInstance(context)
    private val baseApi = retrofitClient.create(BaseApi::class.java)
    val isUploadIconSuccess = MutableLiveData<Boolean>()


    fun doUploadImg(meta: ImageMeta) {
        viewModelScope.launch {
            showLoadDialog.value = true
            try {
                val responseBean = retrofitClient.makeApiCall {
                    val body = meta.file!!.toFileBody("image")
                    baseApi.doUploadImage(body)
                }
                if (responseBean.code == 200) {
                    //设置成功后刷新页面
                    val body =
                        UserAvatarBean(responseBean.data.fileId + ":" + responseBean.data.fileName).toBody()

                    val rBean = retrofitClient.makeApiCall {
                        baseApi.doUpdateUserAvatar(SpUtils.getId(), body)
                    }
                    if (rBean.code == 200) {
                        SpUtils.putAvatar(responseBean.data.fileId + ":" + responseBean.data.fileName)
                        isUploadIconSuccess.value = true
                        showLoadDialog.value = false
                    } else {
                        isUploadIconSuccess.value = false
                        showLoadDialog.value = false
                    }
                } else {
                    isUploadIconSuccess.value = false
                    showLoadDialog.value = false
                }
            } catch (e: Exception) {
                e.printStackTrace()
                isUploadIconSuccess.value = false
                showLoadDialog.value = false
                ExParse.parse(e)
            }

        }
    }
}