package com.tenlionsoft.aimz_k.viewmodel

import android.annotation.SuppressLint
import android.text.Editable
import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.tenlionsoft.aimz_k.adapter.CategoryAdapter
import com.tenlionsoft.aimz_k.dao.DbManager
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.model.ViewState
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import com.tenlionsoft.baselib.widget.AdapterItemClickListener
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.schedulers.Schedulers
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.util.concurrent.TimeUnit

class SearchChatMsgViewModel : BaseViewModel() {
    private val _pwdList = MutableLiveData<List<MsgCategoryBean>?>()
    var adapter: CategoryAdapter = CategoryAdapter(
        _pwdList.value
            ?: emptyList(), this
    )
    val txtMsg = MutableLiveData<String>()
    val clearIcon = MutableLiveData<Boolean>(false)
    var onItemClickListener: AdapterItemClickListener<MsgCategoryBean>? = null

    @SuppressLint("CheckResult")
    fun onTxtChange(s: Editable?) {
        txtMsg.value = s.toString()
        clearIcon.value = s?.isNotEmpty()
        Observable.timer(500, TimeUnit.MILLISECONDS)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe {
                Log.e("SearchChatMsgViewModel", "onTxtChange:${Thread.currentThread().name}")
                if (!txtMsg.value.isNullOrEmpty()) {
                    doSearchChat(txtMsg.value!!)
                } else {
                    pageState.value = ViewState.STATE_SEARCH
                    _pwdList.value = emptyList()
                    adapter.setData(_pwdList.value!!)
                }
            }
    }

    fun clearTxt() {
        txtMsg.value = ""
        clearIcon.value = false
    }

    //搜索聊天记录
    private fun doSearchChat(content: String) {
        viewModelScope.launch {
            val list = doGetList(content)
            Log.e("SearchChatMsgViewModel", "doSearchChat:${list}");
            if (list.isNullOrEmpty()) {
                ToastUtils.normal("未查询到聊天内容")
                pageState.value = ViewState.STATE_EMPTY
            } else {
                _pwdList.value = list
                adapter.setData(_pwdList.value!!)
                pageState.value = ViewState.STATE_SUCCESS
            }
        }
    }

    private suspend fun doGetList(content: String): List<MsgCategoryBean>? {
        return withContext(Dispatchers.IO) {
            val mDao = DbManager.db.msgDao()
            val uId = SpUtils.getId()
            return@withContext mDao.queryContent(content, uId)
        }
    }
}
