package com.tenlionsoft.aimz_k.viewmodel

import android.annotation.SuppressLint
import android.content.Context
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.tenlionsoft.aimz_k.adapter.ContactAdapter
import com.tenlionsoft.aimz_k.model.AddContactBean
import com.tenlionsoft.aimz_k.model.SearchContactBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.contacts.toBody
import com.tenlionsoft.baselib.model.ViewState
import com.tenlionsoft.baselib.net.ExParse
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.ToastUtils
import kotlinx.coroutines.launch

@SuppressLint("StaticFieldLeak")
class SearchContactViewModel(val context: Context) : BaseViewModel() {
    val _contactList = MutableLiveData<List<SearchContactBean>>()
    val txtMsg = MutableLiveData<String>()
    val adapter = ContactAdapter(
        _contactList.value
            ?: emptyList(), this
    )
    val showApplyContentDialog = MutableLiveData<Boolean>(false)
    private val retrofitClient = RetrofitClient.getInstance(context)
    private val userApi = retrofitClient.create(BaseApi::class.java)
    fun onTxtChange(s: CharSequence, start: Int, before: Int, end: Int) {
        txtMsg.value = s.toString()
    }

    //搜索联系人
    fun searchContact(searchText: String) {
        if (searchText.isNotEmpty()) {
            doSearchContact(searchText)
        } else {
            ToastUtils.error("请输入关键字")
        }
    }

    //添加好友
    fun doAddContact(contactListBean: SearchContactBean) {
        viewModelScope.launch {
            try {
                val body = AddContactBean(userId = contactListBean.userId).toBody()
                val isSuccess = retrofitClient.makeApiCall {
                    userApi.doAddContact(body)
                }
                if (isSuccess.code == 200) {
                    //成功
                    ToastUtils.success("请等待好友审核")
                    _contactList.value?.forEach {
                        if (it.userId == contactListBean.userId) it.status = "PENDING"
                    }
                    adapter.setData(_contactList.value!!)
                } else {
                    //失败
                    ToastUtils.error("请稍后重试")
                }
            } catch (e: Exception) {
                e.printStackTrace()
                ExParse.parse(e)
            }
        }
    }

    //通过网络搜索联系人
    private fun doSearchContact(searchText: String) {
        viewModelScope.launch {
            try {
                val list = retrofitClient.makeApiCall {
                    userApi.doSearchContact(searchText)
                }
                if (list.datas.isNotEmpty()) {
                    _contactList.value = list.datas
                    adapter.setData(_contactList.value!!)
                    //刷新页面状态
                    pageState.value = ViewState.STATE_SUCCESS
                } else {
                    //为搜索到
                    pageState.value = ViewState.STATE_EMPTY
                    ToastUtils.success("未搜索到联系人")
                }
            } catch (e: Exception) {
                e.printStackTrace()
                ExParse.parse(e)
            }
        }
    }
}