package com.tenlionsoft.aimz_k.widget

import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import android.util.Log
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import androidx.databinding.BindingAdapter
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.recyclerview.widget.RecyclerView.Adapter
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.request.RequestOptions
import com.bumptech.glide.request.target.CustomTarget
import com.bumptech.glide.request.transition.Transition
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.FileDataBean
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.aimz_k.model.MsgTypeStateEnum
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.model.ImageSize
import com.tenlionsoft.baselib.utils.ImageUtils
import com.tenlionsoft.baselib.utils.TimeUtils
import com.tenlionsoft.baselib.widget.chat.BubbleImageView
import jp.wasabeef.glide.transformations.RoundedCornersTransformation


object BindingUtils {
    @BindingAdapter("android:text")
    @JvmStatic
    fun setText(textView: TextView, bean: MsgCategoryBean?) {
        if (bean == null) {
            textView.text = "匿名用户"
        } else {
            textView.text = if (bean.userNickName.isNullOrEmpty()) "匿名用户" else bean.userNickName
        }
    }

    @BindingAdapter("setLinearLayoutAdapter")
    @JvmStatic
    fun bindAdapterToRecyclerView(recyclerView: RecyclerView, adapter: Adapter<*>) {
        val linearLayoutManager = LinearLayoutManager(recyclerView.context)
        recyclerView.layoutManager = linearLayoutManager
        recyclerView.adapter = adapter
    }

    @BindingAdapter("setLinearLayoutHorizontal")
    @JvmStatic
    fun bindAdapterToRecyclerViewHorizontal(recyclerView: RecyclerView, adapter: Adapter<*>?) {
        val linearLayoutManager =
            LinearLayoutManager(recyclerView.context, LinearLayoutManager.HORIZONTAL, true).apply {
                stackFromEnd = true
            }
        recyclerView.layoutManager = linearLayoutManager
        recyclerView.adapter = adapter
    }

    @BindingAdapter("imageUrl")
    @JvmStatic
    fun bindImgUrl(imageView: ImageView, url: String?) {
        if (url.isNullOrEmpty()) {
            imageView.setImageResource(R.drawable.app_logo_small)
        } else {
            val requestOptions = RequestOptions.bitmapTransform(RoundedCornersTransformation(10, 0))
                .error(R.drawable.ic_user_default).placeholder(R.drawable.ic_user_default)
                .skipMemoryCache(true).diskCacheStrategy(DiskCacheStrategy.NONE)
            Glide.with(imageView.context).load(url).apply(requestOptions).into(imageView)
        }
    }


    @BindingAdapter("imageUserIcon")
    @JvmStatic
    fun userIcon(imageView: ImageView, url: String?) {
        Log.e("BindingUtils", "userIcon:${url}")
        if (url.isNullOrEmpty()) {
            imageView.setImageResource(R.drawable.ic_user_default)
        } else {
            val requestOptions = RequestOptions.bitmapTransform(RoundedCornersTransformation(10, 0))
                .error(R.drawable.ic_user_default).placeholder(R.drawable.ic_user_default)
                .skipMemoryCache(false).diskCacheStrategy(DiskCacheStrategy.NONE)
            val split = url.split(":")
            Glide.with(imageView.context).load(NetConfig.FILE_DOWNLOAD_URL + split[0])
                .apply(requestOptions).into(imageView)
        }
    }


    @BindingAdapter("imgResource")
    @JvmStatic
    fun bindImgUrl(imageView: ImageView, url: Int) {
        imageView.setImageResource(url)
    }

    @BindingAdapter("jsonConvertMsgBody")
    @JvmStatic
    fun jsonConvert(tv: TextView, str: String?) {
        if (str.isNullOrEmpty()) {
            tv.text = ""
        } else {
            val gson = Gson()
            val body = gson.fromJson(str, BodyContent::class.java)
            val contains = body.content!!.contains("fileName")
            if (contains) {
                tv.text = "文件"
            } else {
                tv.text = body.content
            }
        }
    }

    @BindingAdapter("imgUserIconType")
    @JvmStatic
    fun bindCategoryUserIcon(iv: ImageView, bean: MsgCategoryBean?) {
        if (bean == null) {
            iv.setImageResource(R.drawable.ic_user_default)
        } else {
            val requestOptions = RequestOptions.bitmapTransform(RoundedCornersTransformation(10, 0))
                .error(R.drawable.ic_user_default).placeholder(R.drawable.ic_user_default)
                .skipMemoryCache(true).diskCacheStrategy(DiskCacheStrategy.NONE)
            val split = bean.avatar?.split(":")
            if (split != null) {
                Glide.with(iv.context).load(NetConfig.FILE_DOWNLOAD_URL + (split[0]))
                    .apply(requestOptions).into(iv)
            }
        }
    }

    @BindingAdapter("imgUserIconTypeMsg")
    @JvmStatic
    fun bindCategoryUserIconMsg(iv: ImageView, bean: MsgBean?) {
        if (bean == null) {
            iv.setImageResource(R.drawable.ic_user_default)
        } else {
            val requestOptions = RequestOptions.bitmapTransform(RoundedCornersTransformation(10, 0))
                .error(R.drawable.ic_user_default).placeholder(R.drawable.ic_user_default)
                .skipMemoryCache(true).diskCacheStrategy(DiskCacheStrategy.NONE)
            when (bean.msgType) {
                MsgTypeStateEnum.MSG_TO_OTHER_TXT, MsgTypeStateEnum.MSG_TO_OTHER_MOVIE, MsgTypeStateEnum.MSG_TO_OTHER_IMG, MsgTypeStateEnum.MSG_TO_OTHER_FILE -> {
                    val split = bean.senderAvatar?.split(":")
                    if (split != null) {
                        Glide.with(iv.context).load(NetConfig.FILE_DOWNLOAD_URL + (split[0]))
                            .apply(requestOptions).into(iv)
                    } else {
                        iv.setImageResource(R.drawable.ic_user_default)
                    }
                }

                MsgTypeStateEnum.MSG_FROM_OTHER_TXT, MsgTypeStateEnum.MSG_FROM_OTHER_MOVIE, MsgTypeStateEnum.MSG_FROM_OTHER_IMG, MsgTypeStateEnum.MSG_FROM_OTHER_FILE -> {
                    val split = bean.receiverAvatar?.split(":")
                    if (split != null) {
                        Glide.with(iv.context).load(NetConfig.FILE_DOWNLOAD_URL + split[0])
                            .apply(requestOptions).into(iv)
                    } else {
                        iv.setImageResource(R.drawable.ic_user_default)
                    }
                }
            }
        }
    }

    @BindingAdapter("jsonConvertMsgFileIcon")
    @JvmStatic
    fun jsonConvertFileIcon(iv: ImageView, str: String?) {
        if (str.isNullOrEmpty()) {
            iv.visibility = View.GONE
        } else {
            iv.visibility = View.VISIBLE
            val gson = Gson()
            val body = gson.fromJson(str, BodyContent::class.java)
            val contains = body.content!!.contains("fileName")
            if (contains) {
                val fileDataBean = gson.fromJson(body.content, FileDataBean::class.java)
                if (!fileDataBean.fileName.isNullOrEmpty()) {
                    iv.visibility = View.VISIBLE
                    val suffix = fileDataBean.fileName!!.substring(
                        fileDataBean.fileName!!.lastIndexOf("."), fileDataBean.fileName!!.length
                    )
                    var id = R.drawable.ic_audio
                    when (suffix) {
                        ".zip", ".rar", ".7z", ".gz", ".xz" -> iv.setImageResource(R.drawable.ic_zip)
                        ".png", ".jpg", ".jpeg", ".svg", ".gif" -> iv.setImageResource(R.drawable.ic_image)
                        ".doc", ".docx", ".wps", ".wpt" -> iv.setImageResource(R.drawable.ic_word)
                        ".xls", ".xlsx", ".et", ".ett" -> iv.setImageResource(R.drawable.ic_excel)
                        ".pdf" -> iv.setImageResource(R.drawable.ic_pdf)
                        else -> iv.setImageResource(R.drawable.ic_file)
                    }
                } else {
                    iv.visibility = View.GONE
                }
            } else {
                iv.visibility = View.GONE
            }
        }
    }

    @BindingAdapter("jsonConvertMsgFileName")
    @JvmStatic
    fun jsonConvertFileName(tv: TextView, str: String?) {
        if (str.isNullOrEmpty()) {
            tv.text = ""
        } else {
            val gson = Gson()
            val body = gson.fromJson(str, BodyContent::class.java)
            val contains = body.content!!.contains("fileName")
            if (contains) {
                val fileBean = gson.fromJson(body.content, FileDataBean::class.java)
                tv.text = fileBean.fileName
            } else {
                tv.text = body.content
            }
        }
    }

    @BindingAdapter("jsonConvertMsgAudioImg")
    @JvmStatic
    fun jsonConvertAudioCover(iv: ImageView, str: String?) {
        if (str.isNullOrEmpty()) {
            iv.setImageResource(com.tenlionsoft.baselib.R.drawable.ic_img_load_err)
        } else {

            val options = RequestOptions.bitmapTransform(RoundedCornersTransformation(10, 0))
                .error(com.tenlionsoft.baselib.R.drawable.ic_img_load_err)
                .placeholder(com.tenlionsoft.baselib.R.drawable.ic_loading).skipMemoryCache(true)
                .diskCacheStrategy(DiskCacheStrategy.NONE)
            val gson = Gson()
            val bodyContent = gson.fromJson<BodyContent>(str, BodyContent::class.java)

            val fileBean = gson.fromJson(bodyContent.content, FileDataBean::class.java)
            Glide.with(iv.context).load(NetConfig.MAIN_URL + fileBean.fileUrl) // 图片地址
                .apply(options).into(object : CustomTarget<Drawable?>() {
                    override fun onResourceReady(
                        resource: Drawable, transition: Transition<in Drawable?>?
                    ) {
                        val imageSize: ImageSize? =
                            ImageUtils.getImageSize((resource as BitmapDrawable).bitmap)
                        if (imageSize != null) {
                            val imageLP = iv.layoutParams
                            imageLP?.width = imageSize.getWidth()
                            imageLP?.height = imageSize.getHeight()
                            iv.setLayoutParams(imageLP)
                            Glide.with(iv.context).load(resource).apply(options) // 参数
                                .into(iv)
                        }
                    }

                    override fun onLoadCleared(placeholder: Drawable?) {
                    }
                })
        }
    }

    @BindingAdapter("jsonConvertMsgImgBody")
    @JvmStatic
    fun jsonConvertImg(view: BubbleImageView, str: String?) {
        if (str.isNullOrEmpty()) {
            view.setImageResource(com.tenlionsoft.baselib.R.drawable.ic_load_error)
        } else {
            val gson = Gson()
            val body = gson.fromJson(str, BodyContent::class.java)
            val fileData = gson.fromJson(body.content, FileDataBean::class.java)
            val requestOptions = RequestOptions()
                .error(R.drawable.app_logo_small).placeholder(R.drawable.app_logo_small)
                .skipMemoryCache(true).diskCacheStrategy(DiskCacheStrategy.NONE)

            Glide.with(view.context).load(NetConfig.MAIN_URL + fileData.fileUrl)
                .apply(requestOptions).into(view)
        }
    }

    @BindingAdapter("millConvertDate")
    @JvmStatic
    fun millConvertDate(tv: TextView, long: Long?) {
        if (long == null) {
            tv.text = ""
        } else {
            tv.text = TimeUtils.getFriendlyTimeSpanByNow(long)
        }
    }

    //发送中
    @BindingAdapter("isShowLoading")
    @JvmStatic
    fun sending(view: View, status: String?) {
        if (status.isNullOrEmpty()) {
            view.visibility = View.GONE
        } else {
            when (status) {
                ProjectConfig.MSG_SEND_SUCCESS -> view.visibility = View.GONE
                ProjectConfig.MSG_SEND_FAIL -> view.visibility = View.GONE
                ProjectConfig.MSG_SEND_ING -> view.visibility = View.VISIBLE
                else -> view.visibility = View.GONE
            }
        }
    }


    //发送失败
    @BindingAdapter("isShowFail")
    @JvmStatic
    fun sendFail(view: View, status: String?) {
        if (status.isNullOrEmpty()) {
            view.visibility = View.GONE
        } else {
            when (status) {
                ProjectConfig.MSG_SEND_SUCCESS -> view.visibility = View.GONE
                ProjectConfig.MSG_SEND_FAIL -> view.visibility = View.VISIBLE
                ProjectConfig.MSG_SEND_ING -> view.visibility = View.GONE
                else -> view.visibility = View.GONE
            }
        }
    }
}