package com.tenlionsoft.baselib.base

import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.os.Build
import android.view.View
import android.view.WindowManager
import android.view.inputmethod.InputMethodManager
import androidx.core.content.ContextCompat
import com.tenlionsoft.baselib.R
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.SpUtils
import kotlin.system.exitProcess


abstract class BaseActivity : DataBindingActivity() {

    fun reStartApp() {
        SpUtils.removeAll()
        val broadcast = Intent()
        broadcast.setAction(ProjectConfig.A_S_LOGOUT)
        sendBroadcast(broadcast)
        val intent = packageManager.getLaunchIntentForPackage(packageName)
        intent!!.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK or Intent.FLAG_ACTIVITY_NEW_TASK)
        startActivity(intent)
        finish()
    }

    fun exitApp() {
        android.os.Process.killProcess(android.os.Process.myPid());    //获取PID
        exitProcess(0);   //常规java、c#的标准退出法，返回值为0代表正常退出
    }

    fun hideSoftKeyboard() {
        hideSoftKeyboard(currentFocus)
    }

    private fun hideSoftKeyboard(et: View?) {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        et?.postDelayed({ imm.hideSoftInputFromWindow(et.windowToken, 0) }, 100)
    }

    fun showSoftKeyBoard(view: View?) {
        val imm = getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
        view?.postDelayed({
            imm.showSoftInput(view, InputMethodManager.SHOW_IMPLICIT)
        }, 100)
    }

    fun safeExitApp() {
        try {
            // Clear all user data
            SpUtils.removeAll()

            // Send broadcast to stop services
            val broadcast = Intent()
            broadcast.action = ProjectConfig.A_S_LOGOUT
            sendBroadcast(broadcast)

            // Finish all activities
            finishAffinity()

            // Kill the process
            android.os.Process.killProcess(android.os.Process.myPid())

            // Exit the app
            exitProcess(0)
        } catch (e: Exception) {
            e.printStackTrace()
            // Fallback to force exit if safe exit fails
            exitProcess(1)
        }
    }

    fun setStatusBarStyle(transparent: Boolean) {
        val window = window
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            // Add flag to make status bar drawable
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
            if (transparent) {
                // Set transparent status bar
                window.statusBarColor = Color.TRANSPARENT
                // Set dark status bar icons
                window.decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
            } else {
                // Set black status bar
                window.statusBarColor = Color.BLACK
                // Set light status bar icons
                window.decorView.systemUiVisibility = 0
            }
        }
    }

    fun setStatusBarColor(dark: Boolean) {
        val window = window
        if (dark) {
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
            window.statusBarColor = ContextCompat.getColor(this, com.tenlionsoft.baselib.R.color.tr)
        } else {
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
            window.statusBarColor = ContextCompat.getColor(this, R.color.white)
        }

    }
}