package com.tenlionsoft.baselib.base

import android.annotation.SuppressLint
import android.util.Log
import android.view.ViewGroup
import androidx.databinding.ViewDataBinding
import androidx.recyclerview.widget.RecyclerView

/**
 * Adapter基类
 */
abstract class BaseBindingAdapter<D, I : ViewDataBinding>(
    var list: List<D>
) :
    RecyclerView.Adapter<BaseBindingAdapter.BaseViewHolder<I>>() {

    abstract fun getItemBinding(parent: ViewGroup, viewType: Int): I

    abstract fun bindItem(holder: BaseViewHolder<I>, position: Int)


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder<I> {
        val binding = getItemBinding(parent, viewType)
        return BaseViewHolder(binding)
    }


    override fun onBindViewHolder(holder: BaseViewHolder<I>, position: Int) {
        bindItem(holder, position)
        holder.binding.executePendingBindings()
    }

    override fun getItemCount(): Int {
        return list.size
    }

    @SuppressLint("NotifyDataSetChanged")
    fun setData(list: List<D>) {
        this.list = list
        this.notifyDataSetChanged()
    }

    class BaseViewHolder<I : ViewDataBinding>(val binding: I) :
        RecyclerView.ViewHolder(binding.root)
}