package com.tenlionsoft.baselib.net

import android.util.Log
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.utils.SpUtils
import okhttp3.HttpUrl
import okhttp3.HttpUrl.Companion.toHttpUrlOrNull
import okhttp3.Interceptor
import okhttp3.Request
import okhttp3.Response

/**
 * 网络请求拦截器
 */
class BaseUrlInterceptor : Interceptor {

    override fun intercept(chain: Interceptor.Chain): Response {
        val request: Request = chain.request();
        val builder = request.newBuilder();
        val oldHttpUrl = request.url;
        val headers = request.headers("project");
        //公共Token header
        val tokens = request.headers("token")
        if (tokens.isNotEmpty()) {
            builder.removeHeader("token")
            builder.addHeader("X-WG-TOKEN", SpUtils.getToken())
            builder.addHeader("X-WG-TYPE", "APP")
        }

        if (headers.isNotEmpty()) {
            builder.removeHeader("project");
            val headerValue = headers[0];
            var newBaseUrl: HttpUrl? = null;
            if ("user" == headerValue) {
                newBaseUrl = NetConfig.MAIN_URL.toHttpUrlOrNull();
            } else {
                newBaseUrl = oldHttpUrl;
            }
            val newFullUrl: HttpUrl = oldHttpUrl.newBuilder()
                .scheme(scheme = newBaseUrl!!.scheme)
                .host(host = newBaseUrl.host)
                .port(newBaseUrl.port)
                .setPathSegment(0, newBaseUrl.pathSegments[0])
                .build();
            return chain.proceed(builder.url(newFullUrl).build());
        } else {
            return chain.proceed(builder.url(oldHttpUrl).build());
        }
    }
}