package com.tenlionsoft.baselib.net

import okhttp3.MediaType
import okhttp3.ResponseBody
import okio.Buffer
import okio.BufferedSource
import okio.ForwardingSource
import okio.Source
import okio.buffer
import java.io.IOException

class ResponseProgressBody : ResponseBody {
    private var mResponseBody: ResponseBody? = null
    private var mDownloadResponseHandler: DownloadResponseHandler? = null
    private var bufferedSource: BufferedSource? = null

    constructor(
        responseBody: ResponseBody,
        downloadResponseHandler: DownloadResponseHandler
    ) : super() {
        this.mResponseBody = responseBody
        this.mDownloadResponseHandler = downloadResponseHandler
    }


    override fun contentType(): MediaType? {
        return mResponseBody!!.contentType()
    }

    override fun contentLength(): Long {
        return mResponseBody!!.contentLength()
    }

    override fun source(): BufferedSource {
        if (bufferedSource == null) {
            bufferedSource = source(mResponseBody!!.source()).buffer()
        }
        return bufferedSource as BufferedSource
    }

    private fun source(source: Source): Source {
        return object : ForwardingSource(source) {
            var totalBytesRead: Long = 0

            @Throws(IOException::class)
            override fun read(sink: Buffer, byteCount: Long): Long {
                val bytesRead = super.read(sink, byteCount)
                totalBytesRead += (if ((bytesRead != -1L)) bytesRead else 0)
                if (mDownloadResponseHandler != null) {
                    mDownloadResponseHandler!!.onProgress(
                        totalBytesRead,
                        mResponseBody!!.contentLength()
                    )
                }
                return bytesRead
            }
        }
    }
}