package com.tenlionsoft.baselib.utils

import androidx.collection.SimpleArrayMap
import java.util.Locale
import java.util.regex.Pattern

object RegexUtils {
    private val CITY_MAP: SimpleArrayMap<String, String> = SimpleArrayMap()
    const val REGEX_MOBILE_SIMPLE: String = "^[1]\\d{10}$"
    const val REGEX_MOBILE_EXACT: String =
        "^((13[0-9])|(14[5,7])|(15[0-3,5-9])|(16[6])|(17[0,1,3,5-8])|(18[0-9])|(19[0-9]))\\d{8}$"
    const val REGEX_CAR_NUMBER: String =
        "^([京津晋冀蒙辽吉黑沪苏浙皖闽赣鲁豫鄂湘粤桂琼渝川贵云藏陕甘青宁新][A-Z][1-9DF][1-9A-Z]\\d{3}[1-9DF]|[京津晋冀蒙辽吉黑沪苏浙皖闽赣鲁豫鄂湘粤桂琼渝川贵云藏陕甘青宁新][A-Z][\\dA-Z]{5})$"
    const val REGEX_TEL: String =
        "^0\\d{2,3}-?\\d{7,8}$|^((13[0-9])|(14[5,7])|(15[0-9,5-9])|(16[6])|(17[0,1,3,5-8])|(18[0-9])|(19[0-9]))\\d{8}$"
    const val REGEX_ID_CARD15: String = "^[1-9]\\d{7}((0\\d)|(1[0-2]))(([0|1|2]\\d)|3[0-1])\\d{3}$"
    const val REGEX_ID_CARD18: String =
        "^[1-9]\\d{5}[1-9]\\d{3}((0\\d)|(1[0-2]))(([0|1|2]\\d)|3[0-1])\\d{3}([0-9Xx])$"
    const val REGEX_EMAIL: String = "^\\w+([-+.]\\w+)*@\\w+([-.]\\w+)*\\.\\w+([-.]\\w+)*$"
    const val REGEX_URL: String = "[a-zA-z]+://[^\\s]*"
    const val REGEX_ZH: String = "^[\\u4e00-\\u9fa5]+$"
    const val REGEX_USERNAME: String = "^[\\w\\u4e00-\\u9fa5]{6,20}(?<!_)$"
    const val REGEX_DATE: String =
        "^(?:(?!0000)[0-9]{4}-(?:(?:0[1-9]|1[0-2])-(?:0[1-9]|1[0-9]|2[0-8])|(?:0[13-9]|1[0-2])-(?:29|30)|(?:0[13578]|1[02])-31)|(?:[0-9]{2}(?:0[48]|[2468][048]|[13579][26])|(?:0[48]|[2468][048]|[13579][26])00)-02-29)$"
    const val REGEX_IP: String =
        "((2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(2[0-4]\\d|25[0-5]|[01]?\\d\\d?)"

    const val REGEX_DOUBLE_BYTE_CHAR: String = "[^\\x00-\\xff]"
    const val REGEX_BLANK_LINE: String = "\\n\\s*\\r"
    const val REGEX_QQ_NUM: String = "[1-9][0-9]{4,}"
    const val REGEX_CHINA_POSTAL_CODE: String = "[1-9]\\d{5}(?!\\d)"
    const val REGEX_POSITIVE_INTEGER: String = "^[1-9]\\d*$"
    const val REGEX_NEGATIVE_INTEGER: String = "^-[1-9]\\d*$"
    const val REGEX_INTEGER: String = "^-?[1-9]\\d*$"
    const val REGEX_NOT_NEGATIVE_INTEGER: String = "^[1-9]\\d*|0$"
    const val REGEX_NOT_POSITIVE_INTEGER: String = "^-[1-9]\\d*|0$"
    const val REGEX_POSITIVE_FLOAT: String = "^[1-9]\\d*\\.\\d*|0\\.\\d*[1-9]\\d*$"
    const val REGEX_NEGATIVE_FLOAT: String = "^-[1-9]\\d*\\.\\d*|-0\\.\\d*[1-9]\\d*$"
    const val REGEX_PWD: String = "^(?![0-9]+$)(?![a-z]+$)[0-9a-z]{6,}$"
    const val REGEX_PWD_8: String = "^(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z]{8,}$"
    ///////////////////////////////////////////////////////////////////////////
    // If u want more please visit http://toutiao.com/i6231678548520731137
    ///////////////////////////////////////////////////////////////////////////
    /**
     * Return whether input matches regex of simple mobile.
     *
     * @param input The input.
     * @return `true`: yes<br></br>`false`: no
     */
    fun isMobileSimple(input: CharSequence?): Boolean {
        return isMatch(REGEX_MOBILE_SIMPLE, input)
    }

    /**
     * Return whether input matches regex of exact mobile.
     *
     * @param input The input.
     * @return `true`: yes<br></br>`false`: no
     */
    fun isMobileExact(input: CharSequence?): Boolean {
        return isMatch(REGEX_MOBILE_EXACT, input)
    }

    /**
     * Return whether input matches regex of telephone number.
     *
     * @param input The input.
     * @return `true`: yes<br></br>`false`: no
     */
    fun isTel(input: CharSequence?): Boolean {
        return isMatch(REGEX_TEL, input)
    }

    fun isMatch(regex: String?, input: CharSequence?): Boolean {
        return !input.isNullOrEmpty() && Pattern.matches(regex, input)
    }

    fun isEmail(input: CharSequence?): Boolean {
        return isMatch(REGEX_EMAIL, input)
    }

    fun isIDNumber(IDNumber: String?): Boolean {
        if (IDNumber == null || "" == IDNumber) {
            return false
        }
        // 定义判别用户身份证号的正则表达式（15位或者18位，最后一位可以为字母）
        val regularExpression =
            "(^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$)|" + "(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}$)"
        val matches = IDNumber.matches(regularExpression.toRegex())
        //判断第18位校验值
        if (matches) {
            if (IDNumber.length == 18) {
                try {
                    val charArray = IDNumber.toCharArray()
                    //前十七位加权因子
                    val idCardWi = intArrayOf(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2)
                    //这是除以11后，可能产生的11位余数对应的验证码
                    val idCardY = arrayOf("1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2")
                    var sum = 0
                    for (i in idCardWi.indices) {
                        val current = charArray[i].toString().toInt()
                        val count = current * idCardWi[i]
                        sum += count
                    }
                    val idCardLast = charArray[17]
                    val idCardMod = sum % 11
                    return if (idCardY[idCardMod].uppercase(Locale.getDefault()) == idCardLast.toString()
                            .uppercase(
                                Locale.getDefault()
                            )
                    ) {
                        true
                    } else {
                        false
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                    return false
                }
            }
        }
        return matches
    }

    fun isIDCard18Exact(input: CharSequence): Boolean {
        if (isIDCard18(input)) {
            val factor = intArrayOf(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2)
            val suffix = charArrayOf('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2')
            if (CITY_MAP.isEmpty) {
                CITY_MAP.put("11", "北京")
                CITY_MAP.put("12", "天津")
                CITY_MAP.put("13", "河北")
                CITY_MAP.put("14", "山西")
                CITY_MAP.put("15", "内蒙古")

                CITY_MAP.put("21", "辽宁")
                CITY_MAP.put("22", "吉林")
                CITY_MAP.put("23", "黑龙江")

                CITY_MAP.put("31", "上海")
                CITY_MAP.put("32", "江苏")
                CITY_MAP.put("33", "浙江")
                CITY_MAP.put("34", "安徽")
                CITY_MAP.put("35", "福建")
                CITY_MAP.put("36", "江西")
                CITY_MAP.put("37", "山东")

                CITY_MAP.put("41", "河南")
                CITY_MAP.put("42", "湖北")
                CITY_MAP.put("43", "湖南")
                CITY_MAP.put("44", "广东")
                CITY_MAP.put("45", "广西")
                CITY_MAP.put("46", "海南")

                CITY_MAP.put("50", "重庆")
                CITY_MAP.put("51", "四川")
                CITY_MAP.put("52", "贵州")
                CITY_MAP.put("53", "云南")
                CITY_MAP.put("54", "西藏")

                CITY_MAP.put("61", "陕西")
                CITY_MAP.put("62", "甘肃")
                CITY_MAP.put("63", "青海")
                CITY_MAP.put("64", "宁夏")
                CITY_MAP.put("65", "新疆")

                CITY_MAP.put("71", "台湾")
                CITY_MAP.put("81", "香港")
                CITY_MAP.put("82", "澳门")
                CITY_MAP.put("91", "国外")
            }
            if (CITY_MAP.get(input.subSequence(0, 2).toString()) != null) {
                var weightSum = 0
                for (i in 0..16) {
                    weightSum += (input[i].code - '0'.code) * factor[i]
                }
                val idCardMod = weightSum % 11
                val idCardLast = input[17]
                return idCardLast == suffix[idCardMod]
            }
        }
        return false
    }

    private fun isIDCard18(input: CharSequence?): Boolean {
        return isMatch(REGEX_ID_CARD18, input)
    }
}