package com.tenlionsoft.baselib.widget

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.ValueAnimator
import android.animation.ValueAnimator.AnimatorUpdateListener
import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.LinearGradient
import android.graphics.Paint
import android.graphics.Path
import android.graphics.PointF
import android.graphics.RectF
import android.graphics.Shader
import android.os.Build
import android.util.AttributeSet
import android.view.View
import android.view.animation.LinearInterpolator
import androidx.annotation.RequiresApi
import com.tenlionsoft.baselib.R
import com.tenlionsoft.baselib.utils.DensityUtils
import java.text.DecimalFormat
import kotlin.math.max
import kotlin.math.min

class CircleProgress : View {
    private var mContext: Context? = null

    /**
     * 默认进度圆环渐变色值集合
     */
    private var mDefColors = intArrayOf(Color.rgb(0, 187, 238), Color.rgb(0, 34, 255))

    /**
     * 100%进度圆环渐变色值集合
     */
    private val mEndColors = intArrayOf(Color.rgb(255, 196, 0), Color.rgb(255, 110, 77))
    private val mStartColor = Color.rgb(101, 226, 175)
    private val mEndColor = Color.rgb(88, 181, 250)

    // 圆心x坐标
    private var centerX = 0f // 圆心x坐标
    private var centerY = 0f

    // 圆的半径
    private var radius = 0f

    // 进度
    private var mProgress = 0f

    // 当前进度
    private var currentProgress = 0f

    // 圆形进度条底色画笔
    private var circleBgPaint: Paint? = null

    // 圆形进度条进度画笔
    private var progressPaint: Paint? = null

    // 进度条背景颜色
    private var circleBgColor = Color.rgb(225, 229, 232)

    // 进度条颜色
    private var progressColor = Color.RED

    // 默认圆环的宽度
    private val defaultStrokeWidth = 10

    // 圆形背景画笔宽度
    private var circleBgStrokeWidth = defaultStrokeWidth

    // 圆形进度画笔宽度
    private var progressStrokeWidth = defaultStrokeWidth
    private var lineWidth = 0
    private var isDrawCenterProgressText = false
    private var centerProgressTextSize = 23
    private var centerProgressTextColor = Color.BLACK
    private var targetTextSize = 10
    private var targetTextColor = Color.GRAY
    private var targetNumSize = 20
    private var targetNumColor = Color.BLACK
    private var lineColor = Color.GRAY

    // 各种画笔
    private var centerProgressTextPaint = Paint(Paint.ANTI_ALIAS_FLAG)
    private var trainPaint = Paint(Paint.ANTI_ALIAS_FLAG)
    private var linePaint = Paint(Paint.ANTI_ALIAS_FLAG)
    private var targetTextPaint = Paint(Paint.ANTI_ALIAS_FLAG)


    //    private Paint targetNumPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
    // 扇形所在矩形
    private val rectF = RectF()

    // 进度动画
    private var progressAnimator: ValueAnimator? = null

    // 动画执行时间
    private var duration = 1000

    // 动画延时启动时间
    private val startDelay = 500

    private var progressListener: ProgressListener? = null
    private var targetText: String? = null
    private var targetNum = "0"

    //    private lateinit var dp1,dp2, dp4, dp5, dp6, dp7, dp8, dp30
    private var dp1: Int = -1
    private var dp2: Int = -1
    private var dp4: Int = -1
    private var dp5: Int = -1
    private var dp6: Int = -1
    private var dp7: Int = -1
    private var dp8: Int = -1
    private var dp30: Int = -1

    constructor(context: Context) : this(context, null) {

    }

    constructor(context: Context, attr: AttributeSet?) : super(context, attr) {
        mContext = context
        dp1 = DensityUtils.dp2px(mContext!!, 1F)
        dp2 = DensityUtils.dp2px(mContext!!, 2F)
        dp4 = DensityUtils.dp2px(mContext!!, 4F)
        dp5 = DensityUtils.dp2px(mContext!!, 5F)
        dp6 = DensityUtils.dp2px(mContext!!, 6F)
        dp7 = DensityUtils.dp2px(mContext!!, 7F)
        dp8 = DensityUtils.dp2px(mContext!!, 8F)
        dp30 = DensityUtils.dp2px(mContext!!, 30F)
        getAttr(attr)
        initPaint()
    }


    private fun getAttr(attrs: AttributeSet?) {
        val typedArray = mContext!!.obtainStyledAttributes(attrs, R.styleable.CircleProgressBarView)

        circleBgStrokeWidth =
            typedArray.getDimensionPixelOffset(
                R.styleable.CircleProgressBarView_circleBgStrokeWidth,
                defaultStrokeWidth
            )
        progressStrokeWidth =
            typedArray.getDimensionPixelOffset(
                R.styleable.CircleProgressBarView_progressStrokeWidth,
                defaultStrokeWidth
            )

        circleBgColor =
            typedArray.getColor(R.styleable.CircleProgressBarView_circleBgColor, circleBgColor)
        progressColor =
            typedArray.getColor(R.styleable.CircleProgressBarView_progressColor, progressColor)

        duration =
            typedArray.getColor(R.styleable.CircleProgressBarView_circleAnimationDuration, duration)

        isDrawCenterProgressText = typedArray.getBoolean(
            R.styleable.CircleProgressBarView_isDrawCenterProgressText,
            false
        )

        centerProgressTextColor = typedArray.getColor(
            R.styleable.CircleProgressBarView_centerProgressTextColor,
            mEndColor
        )
        centerProgressTextSize =
            typedArray.getDimensionPixelOffset(
                R.styleable.CircleProgressBarView_centerProgressTextSize,
                DensityUtils.sp2px(mContext!!, centerProgressTextSize.toFloat())
            )
        lineWidth = typedArray.getDimensionPixelOffset(
            R.styleable.CircleProgressBarView_lineWidth,
            DensityUtils.dp2px(mContext!!, 1F)
        )
        lineColor = typedArray.getColor(R.styleable.CircleProgressBarView_lineColor, circleBgColor)
        targetNumColor =
            typedArray.getColor(R.styleable.CircleProgressBarView_targetNumColor, targetNumColor)
        targetNumSize = typedArray.getDimensionPixelSize(
            R.styleable.CircleProgressBarView_targetNumSize,
            DensityUtils.sp2px(mContext!!, 20F)
        )
        targetTextSize = typedArray.getDimensionPixelSize(
            R.styleable.CircleProgressBarView_targetTextSize,
            DensityUtils.sp2px(mContext!!, 12F)
        )
        targetTextColor =
            typedArray.getColor(R.styleable.CircleProgressBarView_targetTextColor, targetTextColor)
        targetText = typedArray.getString(R.styleable.CircleProgressBarView_target_text)
        typedArray.recycle()
    }

    fun setCenterProgressTextColor(color: Int) {
        this.centerProgressTextColor = color
        centerProgressTextPaint.color = color
    }


    private fun initPaint() {
        circleBgPaint = getPaint(circleBgStrokeWidth, circleBgColor)

        progressPaint = getPaint(progressStrokeWidth, progressColor)
        trainPaint = getPaint(DensityUtils.dp2px(mContext!!, 1F), progressColor)
        trainPaint.style = Paint.Style.FILL

        linePaint = getPaint(lineWidth, lineColor)

        // 目标文本画笔配置
        targetTextPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        targetTextPaint.textSize = targetTextSize.toFloat()
        targetTextPaint.color = targetTextColor
        targetTextPaint.textAlign = Paint.Align.CENTER

        // 目标数字画笔配置
//        targetNumPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
//        targetNumPaint.setTextSize(targetNumSize);
//        targetNumPaint.setColor(targetNumColor);
//        targetNumPaint.setTextAlign(Paint.Align.CENTER);

        // 中间文本画笔配置
        centerProgressTextPaint = Paint(Paint.ANTI_ALIAS_FLAG)
        centerProgressTextPaint.textSize = centerProgressTextSize.toFloat()
        centerProgressTextPaint.color = centerProgressTextColor
        centerProgressTextPaint.textAlign = Paint.Align.CENTER
    }

    private fun getPaint(strokeWidth: Int, color: Int): Paint {
        val paint = Paint(Paint.ANTI_ALIAS_FLAG)
        paint.strokeWidth = strokeWidth.toFloat()
        paint.color = color
        paint.isAntiAlias = true
        paint.strokeCap = Paint.Cap.ROUND
        paint.style = Paint.Style.STROKE
        return paint
    }

    private var mShader: LinearGradient? = null

    private fun configShader() {
//        Log.d("201801251336", "progress = " + mProgress);
//        int colors[] = {
//                Color.rgb(97, 212, 198), Color.rgb(100, 225, 176),
//                Color.rgb(93, 198, 221), Color.rgb(90, 187, 239)
//        };
//        float positions[] = {
//                0.2f, 0.3f, 0.3f, 0.2f
//        };
        // 清空画笔
        progressPaint!!.setShader(null)
        mShader = if (mProgress >= 100) {
            LinearGradient(
                centerX, 0f, centerX, height.toFloat(),
                mEndColors, null, Shader.TileMode.CLAMP
            )
        } else {
            LinearGradient(
                centerX, 0f, centerX, height.toFloat(),
                mDefColors, null, Shader.TileMode.CLAMP
            )
        }

        progressPaint!!.setShader(mShader)
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        if (progressAnimator != null) {
            progressAnimator!!.cancel()
        }
    }

    private var proAnimIsEnd = false

    private fun initAnimation() {
        proAnimIsEnd = false
        progressAnimator = ValueAnimator.ofFloat(0f, mProgress)
        progressAnimator!!.setDuration(duration.toLong())
        progressAnimator!!.setStartDelay(startDelay.toLong())
        progressAnimator!!.setInterpolator(LinearInterpolator())
        progressAnimator!!.addUpdateListener(AnimatorUpdateListener { valueAnimator: ValueAnimator ->
            val value = valueAnimator.animatedValue as Float
            mProgress = value
            currentProgress = value * 360 / 100
            if (progressListener != null) {
                progressListener!!.currentProgressListener(value)
            }
            invalidate()
        })
        progressAnimator!!.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationEnd(animation: Animator) {
                super.onAnimationEnd(animation)
                proAnimIsEnd = true
                invalidate()
            }
        })
    }

    fun setTargetNum(num: String) {
        targetNum = num
    }

    override fun onSizeChanged(w: Int, h: Int, oldW: Int, oldH: Int) {
        super.onSizeChanged(w, h, oldW, oldH)
        centerX = (w / 2).toFloat()
        centerY = (h / 2).toFloat()

        // 半径再缩小点，给小三角形空出距离
        radius = (min(w.toDouble(), h.toDouble()) / 2 - max(
            circleBgStrokeWidth.toDouble(),
            progressStrokeWidth.toDouble()
        ) - dp4).toFloat()

        rectF[centerX - radius, centerY - radius, centerX + radius] = centerY + radius
        trainPaint.color = circleBgColor

        configShader()
    }

    fun setColors(colors: IntArray) {
        mDefColors = colors
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        //        LogUtil.logD("201801251106", "centerX = " + centerX + "->centerY = "
//                + centerY + "->mProgress = " + mProgress);
        canvas.drawCircle(centerX, centerY, radius, circleBgPaint!!)
        canvas.drawArc(rectF, -90f, currentProgress, false, progressPaint!!)
        if (isDrawCenterProgressText) {
            drawCenterProgressText(canvas, formatNum(mProgress) + "%")
        }
        if (proAnimIsEnd) {
            drawTriangle(canvas)
        }
    }

    private fun drawCenterProgressText(canvas: Canvas, currentProgress: String) {
        val fontMetrics1 = centerProgressTextPaint.fontMetricsInt
        val baseline =
            ((rectF.bottom + rectF.top - fontMetrics1.bottom - fontMetrics1.top) / 2).toInt()
        //文字绘制到整个布局的中心位置
//        canvas.drawLine(centerX - dp30, centerY, centerX + dp30, centerY, linePaint);
//        canvas.drawText(currentProgress, rectF.centerX(), centerY - dp5, centerProgressTextPaint);
        canvas.drawText(
            currentProgress,
            rectF.centerX(),
            rectF.centerY() + dp5,
            centerProgressTextPaint
        )
    }

    /**
     * 绘制三角形
     *
     * @param canvas
     */
    private fun drawTriangle(canvas: Canvas) {
        val path = Path()
        val pf1: PointF
        val pf2: PointF
        val pf3: PointF
        if (mProgress < 49) {
            trainPaint.color = circleBgColor
            pf1 = PointF(centerX - dp8, centerY * 2 - progressStrokeWidth - dp2)
            pf2 = PointF(centerX - dp1, (height - dp2).toFloat())
            pf3 = PointF(centerX + dp6, centerY * 2 - progressStrokeWidth - dp2)
        } else if (mProgress >= 49 && mProgress < 51) {
            val positions: FloatArray
            val pro = (mProgress * 100).toInt()
            positions = if (mProgress >= 49 && pro < 49.2 * 100) {
                floatArrayOf(0.39f, 0.39f)
            } else if (pro >= 49.2 * 100 && pro < 49.4 * 100) {
                floatArrayOf(0.46f, 0.46f)
            } else if (pro >= 49.4 * 100 && pro < 49.6 * 100) {
                floatArrayOf(0.5f, 0.5f)
            } else if (pro >= 49.6 * 100 && pro < 49.8 * 100) {
                floatArrayOf(0.62f, 0.38f)
            } else if (pro >= 49.8 * 100 && pro < 50 * 100) {
                floatArrayOf(0.64f, 0.36f)
            } else if (pro >= 50 * 100 && pro < 50.2 * 100) {
                floatArrayOf(0.7f, 0.3f)
            } else if (pro >= 50.2 * 100 && pro < 50.4 * 100) {
                floatArrayOf(0.75f, 0.25f)
            } else if (pro >= 50.4 * 100 && pro < 50.6 * 100) {
                floatArrayOf(0.8f, 0.2f)
            } else if (pro >= 50.6 * 100 && pro <= 50.8 * 100) {
                floatArrayOf(0.9f, 0.1f)
            } else {
                floatArrayOf(1.0f, 0f)
            }
            val triangleGdt = LinearGradient(
                centerX + dp6,
                0f,
                centerX - dp8,
                0f,
                intArrayOf(Color.rgb(88, 181, 250), Color.rgb(225, 229, 232)),
                positions, Shader.TileMode.CLAMP
            )
            trainPaint.setShader(triangleGdt)
            pf1 = PointF(centerX - dp8, centerY * 2 - progressStrokeWidth - dp2)
            pf2 = PointF(centerX - dp1, (height - dp2).toFloat())
            pf3 = PointF(centerX + dp6, centerY * 2 - progressStrokeWidth - dp2)
        } else {
            trainPaint.setShader(null) // 不要忘记清空着色器，不能设置的color没有效果，会优先显示shader
            if (mProgress >= 100) {
                trainPaint.color = mEndColors[mEndColors.size - 1]
            } else {
                trainPaint.color = mDefColors[mDefColors.size - 1]
            }
            pf1 = PointF(centerX - dp8, centerY * 2 - progressStrokeWidth - dp2)
            pf2 = PointF(centerX - dp1, height.toFloat())
            pf3 = PointF(centerX + dp6, centerY * 2 - progressStrokeWidth - dp2)
        }
        path.moveTo(pf1.x, pf1.y)
        path.lineTo(pf2.x, pf2.y)
        path.lineTo(pf3.x, pf3.y)
        canvas.drawPath(path, trainPaint)
        path.reset()
    }


    fun startProgressAnimation() {
        progressAnimator!!.start()
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    fun pauseProgressAnimation() {
        progressAnimator!!.pause()
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    fun resumeProgressAnimation() {
        progressAnimator!!.resume()
    }

    fun stopProgressAnimation() {
        progressAnimator!!.end()
    }


    /**
     * 传入一个进度值，从0到progress动画变化
     *
     * @param progress
     * @return
     */
    fun setProgressWithAnimation(progress: Float): CircleProgress {
//        if (progress <= 0f || String.valueOf(progress).equals(String.valueOf(Float.NaN))) {
        if (progress <= 0f || java.lang.Float.isNaN(progress)) {
            setCurrentProgress(0f, true)
            return this
        }
        mProgress = progress
        //        if (progress >= 100) {
//            configShader();
//        }else{
        configShader()
        //        }
        initAnimation()
        startProgressAnimation()
        return this
    }

    /**
     * 实时进度，适用于下载进度回调时候之类的场景
     *
     * @param progress
     * @return
     */
    fun setCurrentProgress(progress: Float): CircleProgress {
        mProgress = progress
        currentProgress = progress * 360 / 100
        if (progress >= 100) {
            configShader()
        }
        invalidate()
        return this
    }

    /**
     * 实时进度，适用于下载进度回调时候之类的场景
     *
     * @param progress
     * @param isArrow  是否有小箭头
     * @return
     */
    fun setCurrentProgress(progress: Float, isArrow: Boolean): CircleProgress {
        proAnimIsEnd = isArrow
        setCurrentProgress(progress)
        return this
    }

    /**
     * 格式化数字(保留两位小数)
     *
     * @param money
     * @return
     */
    fun formatNum(money: Float): String {
        if ((money.toInt() * 100) == (money * 100).toInt()) {
            //如果是一个整数
            return money.toInt().toString()
        }
        val format = DecimalFormat("0.00")
        return format.format(money.toDouble())
    }

    interface ProgressListener {
        fun currentProgressListener(currentProgress: Float)
    }

}